<?php


class Zip extends ZipArchive {
    

    public function __construct($zip_name, $zip_dir='./', $overwrite=True)
    {
        $dest_dir = (substr($zip_dir, -1) != '/') ? $zip_dir.'/' : $zip_dir;
        
        if(!self::checkDir($dest_dir))
            return;
        
        if($this->open($dest_dir.$zip_name, ZIPARCHIVE::CREATE | ZIPARCHIVE::OVERWRITE) !== True)
        {
            trigger_error('Zipper :: Impossible de créer le fichier « '.$dest_dir.$zip_name.' ».');
            return;
        }
    }
    

 
    public function add($to_add, $recursive=True)
    {
        if(!is_array($to_add))
            $to_add = array($to_add);
        
        foreach($to_add as $add)
            $this->_add($add, (bool) $recursive);
        
        return $this;
    }
    

    
    public static function unzip($file, $dest_dir='./')
    {
        $zip = new ZipArchive;
        if($zip->open($file) !== True)
        {
            trigger_error('Zipper :: Impossible d\'ouvrir le fichier « '.$file.' ».');
            return False;
        }
        
        if(!self::checkDir($dest_dir))
            return False;
        
        $ret = $zip->extractTo($dest_dir);
        
        $zip->close();
        
        return $ret;
    }
    

    private function _add($to_pack, $recursive)
    {
        if(is_dir($to_pack))
        {
            $to_pack = ($to_pack[ strlen($to_pack)-1] == '/') ? $to_pack : $to_pack.'/';
            $to_pack = (substr($to_pack, 0, 2) == './') ? substr($to_pack, 2) : $to_pack;
            
            $this->addEmptyDir($to_pack);
            
            if((bool) $recursive)
                $this->_exploreAndPack($to_pack);
        }
        else
        {
            if(@file_exists($to_pack)) {
                $this->addFile($to_pack);
			}
            else
                trigger_error('Packer :: « '.$to_pack.' » n\'a pas été trouvé.');
        }
    }
    

    
    private function _exploreAndPack($path)
    {
        $not_to_explore = array('.', '..'); // TODO :: laisser la possibilité à l'user de rajouter des trucs
        $dir = opendir($path);
 
        $actual_dir = '';
        while($item_dir=readdir($dir))
        {
            if(in_array($item_dir, $not_to_explore))
                continue;
            
            if(!is_dir($path.$item_dir)) {
                $this->addFile($path.$item_dir, str_replace('../','',$path.$item_dir));
			}
            else
            {
                $this->addEmptyDir($path.$item_dir);
                
                $this->_exploreAndPack($path.$item_dir.'/');
            }
        }
    }
    

    
    private static function checkDir($dir)
    {
        if(!is_dir($dir))
        {
            if(!mkdir($dir, 0755))
            {
                trigger_error('Zipper :: « '.$dir.' » n\'existe pas et n\'a pas pu être créé.');
                return False;
            }
        }
        else
        {
            if(!is_writable($dir))
            {
                trigger_error('Zipper :: « '.$dir.' » n\'est pas accessible en écriture.');
                return False;
            }
        }
        
        return True;
    }
}


?>